import 'dart:async';
import 'dart:convert';

import 'package:contentsquare/src/modules/gestures/contentsquare_gesture_int_serializer.dart';
import 'package:contentsquare/src/modules/gestures/touch_event_data_json_serializer.dart';
import 'package:contentsquare/src/modules/logger/run_in_error_logger_zone.dart';
import 'package:contentsquare_platform_interface/contentsquare_platform_interface.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'modules/screengraph/painted_render_object_tree.dart';

/// A class exposing Contentsquare SDK's API through its members.
class Contentsquare extends ContentsquarePlatform {
  
  final _a284 = const MethodChannel('contentsquare');

  Contentsquare._a180() {
    
    
    WidgetsFlutterBinding.ensureInitialized();

    _a284.setMethodCallHandler(
      (MethodCall call) async {
        switch (call.method) {
          case 'requestFlutterSnapshot':
            final a285 = call.arguments['rootPath'];
            final a286 = call.arguments['pixelRatio'];

            return a218(
              () {
                return a255(
                  a547: a285 is String ? a285 : '',
                  a548: a286 is double ? a286 : 1,
                );
              },
              a526: true,
            );
        }
      },
    );
  }

  static Contentsquare? _a287;

  /// Returns the [Contentsquare] active [_a287].
  ///
  /// This class follows the singleton pattern so the [_a287] is created the
  /// first time this factory is called.
  factory Contentsquare() => _a287 ??= Contentsquare._a180();

  /// Tracks a screen view with the given name.
  /// You should use this method to track every screen viewed by a user.
  ///
  /// See also
  /// [Screen views in developer documentation](https://docs.contentsquare.com/flutter/#track-screens)
  @override
  Future<void> send(String screenName) {
    return _a284.invokeMethod('send', {'screenName': screenName});
  }

  /// Tracks the given transaction.
  /// Use this method to track purchases made by a user.
  ///
  /// You can use an use the [Currency.toStr] to get the String representation
  /// for the `currency` parameter.
  @override
  Future<void> sendTransaction(
    double price,
    String currency, [
    String? transactionId,
  ]) {
    return _a284.invokeMethod(
      'sendTransaction',
      {'price': price, 'currency': currency, 'transactionId': transactionId},
    );
  }

  /// Tracks a dynamic var.
  ///
  /// You should use either the `stringValue` or the `intValue` parameter
  @override
  Future<void> sendDynamicVar(
    String key, {
    String? stringValue,
    int? intValue,
  }) {
    if (stringValue == null && intValue == null) {
      throw 'stringValue or intValue should be defined';
    }
    if (stringValue != null && intValue != null) {
      throw 'stringValue and intValue cannot be defined at the same time';
    }

    return _a284.invokeMethod(
      'sendDynamicVar',
      {'key': key, 'stringValue': stringValue, 'intValue': intValue},
    );
  }

  /// Opts the current user's device into being tracked.
  ///
  /// When this method is called tracking starts immediately.
  @override
  Future<void> optIn() => _a284.invokeMethod('optIn');

  /// Opts the current user's device out from being tracked.
  ///
  /// Immediately stops tracking the current device. The device will never be
  /// tracked again, unless the app is reinstalled, or you call the `optIn`
  /// method. In such case, the user will be tracked with a new user id and
  /// won't be linked to any previous activity.
  @override
  Future<void> optOut() => _a284.invokeMethod('optOut');

  /// Stops tracking current runtime and wipes user data.
  ///
  /// Any future tracked activity from the current user will have a new user id,
  /// and won't be linked to any past activity from this user. Tracking will
  /// resume on next app launch.
  @override
  Future<void> forgetMe() => _a284.invokeMethod('forgetMe');

  /// The anonymous id used by Contentsquare's SDK to track the current user.
  @override
  Future<String?> getUserId() {
    return _a284.invokeMethod<String>('getUserId');
  }

  /// Stops any further tracking for the current runtime.
  ///
  /// Call this method if you need to pause tracking on some particularly
  /// sensitive part of your app. You can resume tracking at any point by
  /// calling [resumeTracking]. Tracking will also resume on next app launch.
  @override
  Future<void> stopTracking() => _a284.invokeMethod('stopTracking');

  /// Resumes tracking.
  ///
  /// Call this method when you want to resume tracking after having called
  /// [stopTracking]
  @override
  Future<void> resumeTracking() => _a284.invokeMethod('resumeTracking');

  /// Consume touch event.
  ///
  /// Call this method when you want to track a touch event
  @override
  Future<void> sendTouchEvent({
    required ContentsquareGesture type,
    required int version,
    required TouchEventData data,
  }) {
    return _a284.invokeMethod(
      'sendTouchEvent',
      {
        'body': jsonEncode(
          {
            'type': type.a189(),
            'version': version,
            'data': data.a210(),
          },
        ),
      },
    );
  }
}
